#!/usr/bin/python

import serial
import subprocess
import time
import uuid
import os

class Autoloader(object):
    def __init__(self, port='/dev/ttyUSB0'):
        self.s = serial.Serial(port, 9600, timeout=3600)
        time.sleep(1)

    def pickup(self):
        self.s.write('10 1\r')
        return self.waitForCmd()
    def load(self):
        self.s.write('11 1\r\n')
        return self.waitForCmd()
    def unload(self):
        self.s.write('12 1\r\n')
        return self.waitForCmd()
    def putdown(self):
        self.s.write('13 1\r\n')
        return self.waitForCmd()
    def drawerOut(self):
        self.s.write('6 1\r\n')
        return self.waitForCmd()
    def drawerIn(self):
        self.s.write('7 1\r\n')
        return self.waitForCmd()

    def openTray(self):
        cmd = ['eject', '/dev/sr0']
        p = subprocess.Popen(cmd)
        p.wait()
        rv = p.returncode
        time.sleep(3)
        if rv == 0: 
            return True
        else:
            return False

    def closeTray(self):
        cmd = ['eject', '-t', '/dev/sr0']
        p = subprocess.Popen(cmd)
        p.wait()
        rv = p.returncode
        time.sleep(3)
        if rv == 0: 
            return True
        else:
            return False

    def mount(self):
        cmd = ['sudo', 'mount', '/dev/sr0', '/mnt/cdrom']
        p = subprocess.Popen(cmd)
        p.wait()
        rv = p.returncode
        if rv == 0: 
            return True
        else:
            return False

    def umount(self):
        cmd = ['sudo', 'umount', '/mnt/cdrom']
        p = subprocess.Popen(cmd)
        p.wait()
        rv = p.returncode
        if rv == 0: 
            return True
        else:
            return False

    def waitForCmd(self):
        while(True):
            ans = self.s.readline()
            if 'done' in ans:
                return 'done'
            elif 'finished' in ans:
                return 'finished'
            else:
                if(ans.strip() != "" and \
                   ans[0].isdigit() == False and \
                   ans[0].isspace() == False):
                    print "From arduino: {}".format(ans.strip())
            time.sleep(0.01)

    def detectContent(self):
        dirname = '/mnt/cdrom'
        """ Check for svcd
        """
        if os.path.isdir(dirname + "/svcd") or os.path.isdir(dirname + '/vcd'):
            return 'svcd'
        elif os.path.isdir(dirname + "/VIDEO_TS"):
            return 'dvd'
        else:
            return 'data'

    def takePhoto(self, destDir):
        cmd = [os.getcwd() + '/camera.py']
        print cmd, destDir
        p = subprocess.Popen(cmd, cwd=destDir)
        p.wait()

    def createDir(self):
        dirName = str(uuid.uuid1())
        os.mkdir('/srv/' + dirName)
        return '/srv/'+dirName

    def readCd(self, destDir):
        time.sleep(25)
        if self.mount() == False:
            print "Mount failed!!"
            return

        srcType = self.detectContent()
        if srcType == 'data':
            cmd = ['rsync', '-av', '/mnt/cdrom/', destDir]
            p = subprocess.Popen(cmd)
            p.wait()
            rv = p.returncode
        elif srcType == 'dvd':
            cmd = ['makemkvcon', '--minlength=3500', '-r', '--decrypt', \
                    '--directio=true', 'mkv', 'disc:0', 'all', destDir]
            p = subprocess.Popen(cmd)
            p.wait()
            rv = p.returncode
        elif srcType == 'svcd':
            self.umount()
            cmd = ['vcdxrip', '-i', '/dev/sr0']
            p = subprocess.Popen(cmd, cwd=destDir)
            p.wait()
            rv = p.returncode
            self.mount()

        self.umount()


a = Autoloader()
i=1

while True:
    print "============= (Loop {}) ============".format(i)
    i+=1
    print "Take photo..."
    a.drawerIn()
    destDir = a.createDir()
    a.takePhoto(destDir)
    print "Pickup..."
    if a.pickup() == 'finished':
        print "No more discs to load!!"
        break
    print "Open tray..."
    while a.openTray() == False:
        a.closeTray()
        time.sleep(1)
    print "Load..."
    a.load()
    print "Close tray..."
    while a.closeTray() == False:
        a.openTray()
        time.sleep(1)
    print "Read CD..."
    a.readCd(destDir)
    print "Open tray..."
    while a.openTray() == False:
        time.sleep(1)
    print "Unload..."
    a.unload()
    print "Close tray..."
    while a.closeTray() == False:
        a.openTray()
        time.sleep(1)
    print "Put down..."
    a.putdown()
    time.sleep(10)
a.drawerOut()
